//===-- mlir-c/Conversion.h - Conversion passes initialization ----*- C -*-===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM
// Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//
//
// This header declares the registration and creation method for conversion
// passes.
//
//===----------------------------------------------------------------------===//

#ifndef MLIR_C_CONVERSION_H
#define MLIR_C_CONVERSION_H

#include "mlir-c/Support.h"

#include "mlir/Conversion/Passes.capi.h.inc"

#endif // MLIR_C_CONVERSION_H
